/*    generat.c - library module for PWGEN program
**    Copyright (C) 1999  Adel I. Mirzazhanov <adel@altair.alatau.kz>
**
**    This program is free software; you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation; either version 2 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program; if not, write to the Free Software
**    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
/*==========================================================
** NAME		generat.c
** TYPE		LIBRARY PROGRAM MODULE
** DESCRIPTION	password generation module of PWGEN program
** INPUT	NONE
** OUTPUT	NONE
** RETURN	NONE
**
** STARTTED AT	07/07/99
** AUTHOR	ADEL I. MIRZAZHANOV
** ENDED AT	DD/MM/YY
==========================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <pwd.h>
#include <unistd.h>
#include "pwgen.h"

char numeric[10] = { '1', '2', '3', '4', '5', '6', '7', '8', '9', '0' };
char cleters[26] = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
		     'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T',
		     'U', 'V', 'W', 'X', 'Y', 'Z' };
char leters[26]  = { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j',
		     'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't',
		     'u', 'v', 'w', 'x', 'y', 'z' };
unsigned short int special [32] = { 33, 34, 35, 36, 37, 38, 39, 40, 41, 42,
				    43, 44, 45, 46, 47, 58, 59, 60, 61, 62,
				    63, 64, 91, 92, 93, 94, 95, 96, 123, 124,
				    125, 126 };

/*==========================================================
** NAME		gen_symbol
** TYPE		FUNCTION
** DESCRIPTION	char gen_symbol(unsigned short int symbol_class);
** INPUT	symbol_class
		    S_NUM	1
		    S_SPC	2
		    S_CLT	3
		    S_LET	4
** OUTPUT	generates a symbol for password;
** RETURN	char - PASSWORD SYMBOL
**
** STARTTED AT	07/07/99
** AUTHOR	ADEL I. MIRZAZHANOV
** ENDED AT	DD/MM/YY
==========================================================*/
char gen_symbol(unsigned short int symbol_class)
    {
     int i = 0;
     if(symbol_class != S_NUM && symbol_class != S_SPC &&
        symbol_class != S_CLT && symbol_class != S_LET)
	{
	 fprintf(stderr,"\ngen_symbol: wrong symbol_class\n");
	 exit(-1);
	}
     switch(symbol_class)
         {
	  case S_NUM:
	      i = 1+(int)(9.0*rand()/(RAND_MAX+1.0));
	      return(numeric[i]);
	  case S_SPC:
	      i = 1+(int)(31.0*rand()/(RAND_MAX+1.0));
	      return(special[i]);
	  case S_CLT:
	      i = 1+(int)(25.0*rand()/(RAND_MAX+1.0));
	      return(cleters[i]);
	  case S_LET:
	      i = 1+(int)(25.0*rand()/(RAND_MAX+1.0));
	      return(leters[i]);
	 }
	 return(255);
    }

/*==========================================================
** NAME		gen_password
** TYPE		FUNCTION
** DESCRIPTION	void gen_password(char* password_string, int length, unsigned int pass_mode)
** INPUT	char* password_string
**		int length
**		unsigned short int pass_mode
**
** OUTPUT	generates a password;
** RETURN	NONE
**
** STARTTED AT	07/07/99
** AUTHOR	ADEL I. MIRZAZHANOV & ELENA YUZHANINA
** ENDED AT	DD/MM/YY
==========================================================*/
void gen_password(char* password_string, int length, unsigned int pass_mode)
    {
     int i = 0;
     int j = 0;
     char* str_pointer;
     int random_weight[4]= {0, 0, 0, 0};
     int max_weight = 0;
     int max_weight_element_number = 0;
     
     if(length > 256)
         {
	  fprintf(stderr,"\ngen_password: password length is too big\n");
	  exit(-1);
	 }
     
     str_pointer = password_string;

     for(i=0; i<=length; i++)
        {

/* Asign random weight in weight array if mode is present*/
	 
         if((pass_mode & S_NUM) > 0)
	    random_weight[0] = 1+(int)(1024.0*rand()/(RAND_MAX+1.0));
	 if((pass_mode & S_SPC) > 0)
	    random_weight[1] = 1+(int)(1024.0*rand()/(RAND_MAX+1.0));
	 if((pass_mode & S_CLT) > 0)
	    random_weight[2] = 1+(int)(1024.0*rand()/(RAND_MAX+1.0));
	 if((pass_mode & S_LET)  > 0)
	    random_weight[3] = 1+(int)(1024.0*rand()/(RAND_MAX+1.0));

/* Find an element with maximum weight*/

    	 for(j=0; j<=3; j++)
	     if(random_weight[j] > max_weight)
	         {
	          max_weight = random_weight[j];
		  max_weight_element_number = j;
		 }
/* Generate password symbol for mode with maximum weight*/
	 switch(max_weight_element_number)
	     {
	      case 0:
	          *str_pointer = gen_symbol(S_NUM);
		  str_pointer++;
		  break;
	      case 1:
	          *str_pointer = gen_symbol(S_SPC);
		  str_pointer++;
		  break;
	      case 2:
	          *str_pointer = gen_symbol(S_CLT);
		  str_pointer++;
		  break;
	      case 3:
	          *str_pointer = gen_symbol(S_LET);
		  str_pointer++;
		  break;
	     }
         max_weight = 0;
         max_weight_element_number = 0;
         for(j=0; j<=3; j++)
             random_weight[j] = 0;
	}
    }

/*==========================================================
** NAME		gen_rangom_seed
** TYPE		FUNCTION
** DESCRIPTION	int gen_rangom_seed(void)
** INPUT	NONE
**
** OUTPUT	NONE
** RETURN	random seed for srand()
**
** STARTTED AT	07/07/99
** AUTHOR	ADEL I. MIRZAZHANOV & ELENA YUZHANINA
** ENDED AT	DD/MM/YY
==========================================================*/
int gen_random_seed(void)
    {
     return((int)time(NULL));
    }

/*==========================================================
** NAME		get_random_seq
** TYPE		FUNCTION
** DESCRIPTION	int get_random_seq(void)
** INPUT	NONE
**
** OUTPUT	NONE
** RETURN	random seed for srand()
**
** STARTTED AT	07/07/99
** AUTHOR	ADEL I. MIRZAZHANOV & ELENA YUZHANINA
** ENDED AT	DD/MM/YY
==========================================================*/
int get_random_seq(void)
    {
     char *user_random_string;
     int seed = 0;
     
     printf ("\nEnter up to 128 random symbols then press [Enter]\n");
     user_random_string = getpass("Random string:>");
     while( *user_random_string != 0)
         {
	  seed = (int)(seed*((int)*user_random_string));
	  user_random_string++;
	 }
     return(seed);
    }